"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.subscriptionDeleteParamsParser = exports.subscriptionUpdateResponseParser = exports.subscriptionUpdateParamsParser = exports.subscriptionUpdateInputParser = exports.subscriptionCreateResponseParser = exports.subscriptionCreateInputParser = exports.subscriptionOriginTypeParser = exports.subscriptionRecurringAndStatusParser = exports.subscriptionApiRecurringParser = exports.subscriptionStatusParser = exports.subscriptionAmountUnitParser = exports.subscriptionApiAmountParser = exports.subscriptionUnitParser = exports.subscriptionAmountParser = exports.subscriptionRecurringParser = exports.subscriptionGetOneParamsParser = exports.subscriptionListResponseParser = exports.subscriptionListQueryParser = exports.subscriptionApiSrcDstCompositeParser = exports.subscriptionApiSrcDstCompositeParserOptional = exports.subscriptionApiDstCompositeParser = exports.subscriptionApiSrcCompositeParser = exports.SEPARATOR = void 0;
const zod_1 = __importDefault(require("zod"));
const queryParams_1 = require("../common/parsers/queryParams");
const constants_1 = require("./constants");
const types_1 = require("../utils/types");
const parsers_1 = require("../stripe/parsers");
const parsers_2 = require("../currencies/parsers");
exports.SEPARATOR = '::'; // just coincides with UUID_SEPARATOR
// "api src" - looks just like a normal graph src but is not graph src; abstractions are just similar
// (in truth, its because the older api already works like that but we'd like to both move from the graph terminology and keep compatibility)
const makeApiIdParser = (literals) => {
    const makeType = (literal) => zod_1.default.object({
        type: zod_1.default.literal(literal),
        id: zod_1.default.string().transform((0, types_1.cast)()),
    });
    // to procure type
    const first = makeType(literals[0]);
    return zod_1.default.preprocess((uuid) => {
        // no throws in preprocess https://github.com/colinhacks/zod/issues/1460
        if (typeof uuid !== 'string') {
            console.error(`Expected string, got ${typeof uuid}: ${uuid}`);
            return uuid;
        }
        const [type, id] = uuid.split(exports.SEPARATOR);
        return { type, id: id };
    }, 
    // trick zod https://github.com/colinhacks/zod/issues/1442
    zod_1.default.discriminatedUnion('type', [first, ...literals.slice(1).map(makeType)]));
};
const srcLiterals = [constants_1.SUBSCRIPTION_USER_API_LITERAL_TYPE];
exports.subscriptionApiSrcCompositeParser = makeApiIdParser(srcLiterals);
const dstLiterals = [constants_1.SUBSCRIPTION_CHANNEL_API_LITERAL_TYPE];
exports.subscriptionApiDstCompositeParser = makeApiIdParser(dstLiterals);
exports.subscriptionApiSrcDstCompositeParserOptional = zod_1.default.object({
    src: exports.subscriptionApiSrcCompositeParser.optional(),
    dst: exports.subscriptionApiDstCompositeParser.optional(),
});
exports.subscriptionApiSrcDstCompositeParser = zod_1.default.object({
    src: exports.subscriptionApiSrcCompositeParser,
    dst: exports.subscriptionApiDstCompositeParser,
});
exports.subscriptionListQueryParser = (0, queryParams_1.makeLimitOffsetParser)(queryParams_1.queryLimitParser, queryParams_1.queryOffsetParser).merge(exports.subscriptionApiSrcDstCompositeParserOptional);
exports.subscriptionListResponseParser = zod_1.default.any( /*stub*/).transform((0, types_1.cast)());
exports.subscriptionGetOneParamsParser = zod_1.default.object({
    [constants_1.SUBSCRIPTION_ID_PARAM_NAME]: parsers_1.stripeSubscriptionIdParser,
});
exports.subscriptionRecurringParser = zod_1.default.boolean();
exports.subscriptionAmountParser = parsers_2.currencyAmountParser;
exports.subscriptionUnitParser = parsers_2.currencyParser;
exports.subscriptionApiAmountParser = zod_1.default.object({
    amt: exports.subscriptionAmountParser,
});
exports.subscriptionAmountUnitParser = zod_1.default
    .object({
    unit: exports.subscriptionUnitParser,
})
    .merge(exports.subscriptionApiAmountParser);
exports.subscriptionStatusParser = zod_1.default.enum(constants_1.SUBSCRIPTION_STATUSES);
exports.subscriptionApiRecurringParser = zod_1.default.object({
    recurring: exports.subscriptionRecurringParser,
});
exports.subscriptionRecurringAndStatusParser = zod_1.default
    .object({
    status: exports.subscriptionStatusParser,
})
    .merge(exports.subscriptionApiRecurringParser);
exports.subscriptionOriginTypeParser = zod_1.default.enum(constants_1.SUBSCRIPTION_ORIGIN_TYPES);
// "api" subscription shape; not for use to parse real stripe subscriptions
exports.subscriptionCreateInputParser = zod_1.default
    .object({
// // TODO camelcase; used?
// origin_type: subscriptionOriginTypeParser,
// // TODO camelcase; used?
// origin_id: subscriptionOriginIdParser,
})
    .merge(exports.subscriptionApiSrcDstCompositeParser)
    .merge(exports.subscriptionAmountUnitParser)
    .merge(exports.subscriptionApiRecurringParser);
exports.subscriptionCreateResponseParser = zod_1.default.any( /*stub*/).transform((0, types_1.cast)());
exports.subscriptionUpdateInputParser = exports.subscriptionApiAmountParser.partial();
exports.subscriptionUpdateParamsParser = exports.subscriptionGetOneParamsParser;
exports.subscriptionUpdateResponseParser = exports.subscriptionCreateResponseParser;
exports.subscriptionDeleteParamsParser = exports.subscriptionGetOneParamsParser;
