import z from 'zod';
import { SUBSCRIPTION_CHANNEL_API_LITERAL_TYPE, SUBSCRIPTION_ORIGIN_TYPES, SUBSCRIPTION_PROVIDERS, SUBSCRIPTION_STATUSES, SUBSCRIPTION_USER_API_LITERAL_TYPE } from './constants';
import { StripeSubscriptionId } from '../stripe/types';
import { Currency } from '../currencies/types';
import { SEPARATOR, subscriptionApiDstCompositeParser, subscriptionApiSrcCompositeParser, subscriptionCreateInputParser, subscriptionCreateResponseParser, subscriptionDeleteParamsParser, subscriptionListQueryParser, subscriptionListResponseParser, subscriptionUpdateInputParser, subscriptionUpdateParamsParser, subscriptionUpdateResponseParser } from './parsers';
import { UserId } from '../user/types';
import { ChannelIdOfType, ChannelType } from '../channel/types';
export declare type SubscriptionId = StripeSubscriptionId;
export declare type SubscriptionProvider = typeof SUBSCRIPTION_PROVIDERS[number];
export declare type SubscriptionStatus = typeof SUBSCRIPTION_STATUSES[number];
export declare type SubscriptionOriginType = typeof SUBSCRIPTION_ORIGIN_TYPES[number];
declare type SubscriptionOriginIdPerTypeMapping = {
    paypal: never;
    stripe: StripeSubscriptionId;
};
export declare type SubscriptionOriginIdForType<T extends SubscriptionOriginType> = SubscriptionOriginIdPerTypeMapping[T];
export declare type ApiSubscription = {
    amt: number;
    unit: Currency;
    status: SubscriptionStatus;
    client_secret?: string;
    src: PassportIssueSrc;
    dst: PassportIssueDst;
};
export declare type ApiSubscriptionSrcComposite = z.infer<typeof subscriptionApiSrcCompositeParser>;
export declare type ApiSubscriptionDstComposite = z.infer<typeof subscriptionApiDstCompositeParser>;
export declare type SubscriptionListApiQuery = z.infer<typeof subscriptionListQueryParser>;
export declare type SubscriptionListApiResponse = z.infer<typeof subscriptionListResponseParser>;
export declare type SubscriptionUpdateApiParams = z.infer<typeof subscriptionUpdateParamsParser>;
export declare type SubscriptionUpdateApiBody = z.infer<typeof subscriptionUpdateInputParser>;
export declare type SubscriptionUpdateApiResponse = z.infer<typeof subscriptionUpdateResponseParser>;
export declare type SubscriptionCreateApiBody = z.infer<typeof subscriptionCreateInputParser>;
export declare type SubscriptionCreateApiResponse = z.infer<typeof subscriptionCreateResponseParser>;
export declare type SubscriptionDeleteApiParams = z.infer<typeof subscriptionDeleteParamsParser>;
export declare type PassportIssueSrc = `${typeof SUBSCRIPTION_USER_API_LITERAL_TYPE}${typeof SEPARATOR}${UserId}`;
export declare type PassportIssueDst<T extends ChannelType = ChannelType> = `${typeof SUBSCRIPTION_CHANNEL_API_LITERAL_TYPE}${typeof SEPARATOR}${ChannelIdOfType<T>}`;
declare enum PassportIssueBrand {
    _ = ""
}
export declare type PassportIssueId = string & PassportIssueBrand;
export declare type WithPassportId = {
    id: PassportIssueId;
};
export declare type PassportCore = {
    src: PassportIssueSrc;
    dst: PassportIssueDst;
    issuedAt: number;
    expiresAt: number;
};
export declare type Passport = PassportCore & WithPassportId;
export {};
